#include <avr/io.h>
#include <stdint.h>
#include "joystick.h"
#include "hd44780.h"
#include "other.h"
#include <avr/pgmspace.h>
#include "registerstrings.h"
#include <stdlib.h>
#include <util/delay.h>
#include "usart.h"

uint8_t		joystick_selection 	= 0x00;					// Auswahl Taste gedrckt
uint8_t		level_zero			= 0x00;					// Auswhal von Level 0
uint8_t		level_one			= 0x00;					// Auswahl von Level 1



/*******************************************************
	Funktion :		Initialisiert Joystick
	Eigenschaften:	PA0, PA1 und PA2 als Eingang
	bergabewert:	-
	Rckgabewert:	-
********************************************************/
void joystick_init(void)
{
	DDRA &= (0 << DDA2) & (0 << DDA1) & (0 << DDA0);	// Stelle PA0-PA2 als Eingang ein (Joystick)

	DDRD |= (1 << PD2) | (1 << PD3);					// Stelle die LEDS ein (2 = rot, 3 = grn);
	PORTD &= (0 << PD2);								// ausschalten einer LED (rote)
	PORTD |= (1 << PD3);								// einschalten einer LED (grne)
}



/*******************************************************
	Funktion :		Schaue ob Joystick gedrckt wurde
	Eigenschaften:	-
	bergabewert:	-
	Rckgabewert:	1 Byte, (1 = gedrckt, 0 = nicht gedrckt)
********************************************************/
uint8_t joystick_button_pressed(void)
{
    if (PINA & (1<<PA2))
    {
        // Pin wurde auf Masse gezogen => 100ms warten 
        _delay_ms(50);  
        _delay_ms(50); 
        if (!(PINA & (1<<PA2)))
        {
            // Anwender Zeit zum Loslassen des Tasters geben => 100ms wartne
            _delay_ms(50);
            _delay_ms(50); 
            return 1;		// Taster gedrckt
        }
    }
	return 0;				// Taster nicht gedrckt
}




/*******************************************************
	Funktion :		Messe einen ADC Kanal
	Eigenschaften:	Fhrt 4 Messungen durch und liefert Mittelwert
	bergabewert:	struct time
	Rckgabewert:	-
********************************************************/
uint16_t joystick_adc(uint8_t port)
{
	uint8_t i;
	uint16_t result;
 
	ADMUX = port;                      	// Kanal whlen
	ADMUX |= (1<<REFS1) | (1<<REFS0); 	// interne Referenzspannung benutzen
 
	ADCSRA = (1<<ADEN) | (1<<ADPS1) | (1<<ADPS0);    	// Frequenzvorteiler setzen (auf 8) + ADC aktivieren
 
	// nach Aktivieren ein "Dummy-Readout" machen
	// also einen Wert lesen und verwerfen, um warmlaufen zu lassen

	ADCSRA |= (1<<ADSC);              	// ADC-Wandlung 
	while ( ADCSRA & (1<<ADSC) ) {
		;     							// auf Abschluss der Konvertierung warten 
	}

	result = ADCW;  // ADCW muss einmal gelesen werden,
	                // sonst wird Ergebnis der nchsten Wandlung
	                // nicht bernommen.
 
	// Eigentliche Messung - Mittelwert aus 4 aufeinanderfolgenden Wandlungen
	result = 0; 
	for( i=0; i<4; i++ )
	{
		ADCSRA |= (1<<ADSC);            // eine Wandlung "single conversion"
		while ( ADCSRA & (1<<ADSC) ) {
			;  							// auf Abschluss der Konvertierung warten
		}
		result += ADCW;		    		// Wandlungsergebnisse aufaddieren
	}
	
	ADCSRA &= ~(1<<ADEN);             	// ADC deaktivieren
 
	result /= 4;                     	// Summe durch vier teilen = arithm. Mittelwert
 
	return result;
}




/*******************************************************
	Funktion :		Joystick Messung
	Eigenschaften:	reagiert auf Joystick bettigung
	bergabewert:	Messlnge, Errorcode, aktuelle Zeit, Messmaske low, Messmaske high, Messdaten
	Rckgabewert:	-
********************************************************/
void joystick_measure(uint8_t measured_length, uint8_t errorcode, struct time t,uint32_t measurement_mask_low, uint32_t measurement_mask_high, uint32_t *measured)
{
	uint16_t	joystick_rightleft 	= 0x00;					// Auswahl Rechts-Links von Joystick
	uint16_t	joystick_updown		= 0x00;					// Auswahl Rauf-Runter von Joystick
	uint8_t		measured_counter	= 0x00;					// Zhlt die gemessenen Werte hoch (Die Stelle in der Bitmaske)
	uint8_t		measured_results	= 0x00;					// Gibt an, wieviele Werte schon gefunden wurden (Der wievielte 1er bei der Bitmaske)
	
	
	// -------------------------- Hole Werte von ADC --------------------------
	joystick_rightleft = joystick_adc(0);					// Abfrage ob Rechts/Links gedrckt
	joystick_updown = joystick_adc(1);						// Abfrage ob Rauf/Runter gedrckt


	/*
	// -------------------- Wurde Joystick-Knopf gedrckt ---------------------
	if(joystick_button_pressed())							// Abfrage ob Taster gedrckt wurde
	{
		if(joystick_selection == 1)							// Wert ber der Grenze?
		{
			joystick_selection = 0;							// auf Ebene 0 setzen
			level_one = 0;									// Ebene 1 Auswahl zurcksetzen
		}
		else
		{
			joystick_selection = 1;							// auf Ebene 1 setzen
			level_one = 0;									// Ebene 1 Auswahl zurcksetzen
		}	
	}
	*/

	// -------------- Wurde Joystick nach oben  oder unten gedrckt -------------
	if(joystick_updown <= JOYSTICK_IS_UP)					// wurde nach oben gedrckt?
	{
		joystick_selection = 0;								// Gehe auf Ebene 0 zurck
		level_one = 0;										// Ebene 1 Auswahl zurcksetzen
	}
	else if(joystick_updown >= JOYSTICK_IS_DOWN)
	{
		joystick_selection = 1;							// auf Ebene 1 setzen
		level_one = 0;									// Ebene 1 Auswahl zurcksetzen
	}
	
	_delay_ms(250);

	// --------------- Wurde Joystick nach rechts/links gedrckt ---------------
	if(joystick_selection == 0)								// Befinden man sich auf erster Ebene?
	{
		// ------------------- Berechnung von level_zero -------------------

		if(joystick_rightleft >= JOYSTICK_IS_RIGHT)			// Wurde nach rechts geklickt?
		{
			if(level_zero >= DISPLAY_LEVEL_ZERO_MAX)		// Ist man am rechten Rand?
			{
				level_zero = 0;								// Setze wieder auf linken Rand zurck
			}
			else
			{
				level_zero++;								// Gehe eins nach rechts
			}
		}
		else if(joystick_rightleft <= JOYSTICK_IS_LEFT)		// Wurde nach links geklickt?
		{
			if(level_zero <= 0)								// Ist man am linken Rand?
			{
				level_zero = DISPLAY_LEVEL_ZERO_MAX;		// Setze wieder auf rechten Rand zurck
			}
			else
			{
				level_zero--;								// Gehe eins nach links
			}
		}
		
		// ------------------- Ausgabe von zero_level -------------------
		switch(level_zero)
		{
			case LEVEL_ZERO_STATUS:
			{
				display_write_text("Status", NO_LINE_TWO,10);
			} break;

			case LEVEL_ZERO_MEASUREVALUES:
			{
				display_write_text("Messwerte", NO_LINE_TWO,10);
			} break;

			case LEVEL_ZERO_TIME:
			{
				display_write_text("Zeit", NO_LINE_TWO,10);
			} break;

			case LEVEL_ZERO_CREDITS:
			{
				display_write_text("Credits", NO_LINE_TWO,10);
			} break;
		}
	}
	else if(joystick_selection == 1)						// Befindet man sich auf zweiter Ebene?
	{
		switch(level_zero)
		{
			case LEVEL_ZERO_STATUS:
			{
				// ------------------- Berechnung von level_one -------------------
				// Tue nichts, weil nur 1 Auswahlmglichkeit vorhanden ist
				
				// -------------------- Ausgabe von level_one ---------------------
				display_write_text("Status:",errorcode,16);

			} break;

			case LEVEL_ZERO_MEASUREVALUES:
			{
				// ------------------- Berechnung von level_one -------------------
				if(joystick_rightleft >= JOYSTICK_IS_RIGHT)	// Wurde nach rechts geklickt?
				{
					if(level_one >= (measured_length-1))	// Ist man am rechten Rand? (-1 weil man mit 0 beginnt zu zhlen)
					{
						level_one = 0;						// Setze wieder auf linken Rand zurck
					}
					else
					{
						level_one++;						// Gehe eins nach rechts
					}	
				}
				else if(joystick_rightleft <= JOYSTICK_IS_LEFT)	// Wurde nach links geklickt?
				{
					if(level_one <= 0)						// Ist man am linken Rand?
					{
						level_one = (measured_length-1);	// Setze wieder auf rechten Rand zurck (-1 weil mit 0 zu zhlen begonnen wird)
					}
					else
					{
						level_one--;						// Gehe eins nach links
					}
				}

				// -------------------- Ausgabe von level_one ---------------------
				
				// Berechnung von bentigtem Registerstrings-Index
				for(int i = 0; i < 32; ++i)							// Durchlaufen aller Stellen
				{
					if(((measurement_mask_low >> i) & 1) == 1)		// Kontrolle ob an der Stelle 1 steht
					{
						if(level_one == measured_results)			// Wurde die Stelle schon gefunden?
						{
							goto Ende;								// Sprung raus aus beiden Schleifen
						}
						measured_results++;
					}
					measured_counter++;								// Erhhe, da eine weitere "1" gefunden wurde
				}

				for(int i = 0; i < 7; ++i)							// Durchlauf der verbleibenden Stellen
				{
					if(((measurement_mask_high >> i) & 1) == 1)		// Kontrolle ob an der Stelle 1 steht
					{	
						if(level_one == measured_results)			// Wurde die Stelle schon gefunden?
						{
							break;									// Verlasse die Schleife
						}
						measured_results++;
					}
					measured_counter++;								// Erhhe, da eine weitere "1" gefunden wurde
				}

				Ende:

				// Ausgabe
				display_write_text(registernames[measured_counter], measured[level_one],16);
			} break;

			case LEVEL_ZERO_TIME:
			{

				// ------------------- Berechnung von level_one -------------------
				if(joystick_rightleft >= JOYSTICK_IS_RIGHT)	// Wurde nach rechts geklickt?
				{
					if(level_one >= DISPLAY_TIME_MAX)		// Ist man am rechten Rand?
					{
						level_one = 0;						// Setze wieder auf linken Rand zurck
					}
					else
					{
						level_one++;						// Gehe eins nach rechts
					}
				}
				else if(joystick_rightleft <= JOYSTICK_IS_LEFT)	// Wurde nach links geklickt?
				{
					if(level_one <= 0)						// Ist man am linken Rand?
					{
						level_one = DISPLAY_TIME_MAX;		// Setze wieder auf rechten Rand zurck
					}
					else
					{
						level_one--;						// Gehe eins nach links
					}
				}

				// -------------------- Ausgabe von level_one ---------------------
				switch(level_one)
				{
					case LEVEL_ONE_TIME_SECONDS:
					{
						display_write_text("Sekunde", t.seconds,10);
					} break;

					case LEVEL_ONE_TIME_MINUTES:
					{
						display_write_text("Minute", t.minutes,10);
					} break;

					case LEVEL_ONE_TIME_HOURS:
					{
						display_write_text("Stunde", t.hours,10);
					} break;

					case LEVEL_ONE_TIME_DATE:
					{
						display_write_text("Tag", t.date,10);
					} break;

					case LEVEL_ONE_TIME_MONTH:
					{
						display_write_text("Monat", t.month,10);
					} break;

					case LEVEL_ONE_TIME_YEAR:
					{
						display_write_text("Jahr", t.year,10);
					} break;
				}

			} break;

			case LEVEL_ZERO_CREDITS:
			{

				// ------------------- Berechnung von level_one -------------------
				if(joystick_rightleft >= JOYSTICK_IS_RIGHT)	// Wurde nach rechts geklickt?
				{
					if(level_one >= DISPLAY_CREDITS_MAX)	// Ist man am rechten Rand?
					{
						level_one = 0;						// Setze wieder auf linken Rand zurck
					}
					else
					{
						level_one++;						// Gehe eins nach rechts
					}
				}
				else if(joystick_rightleft <= JOYSTICK_IS_LEFT)	// Wurde nach links geklickt?
				{
					if(level_one <= 0)						// Ist man am linken Rand?
					{
						level_one = DISPLAY_CREDITS_MAX;	// Setze wieder auf rechten Rand zurck
					}
					else
					{
						level_one--;						// Gehe eins nach links
					}
				}

				// -------------------- Ausgabe von level_one ---------------------
				switch(level_one)
				{
					case LEVEL_ONE_VERSION_NR:
					{
						display_write_text("Version:", 1,10);
					} break;

					case LEVEL_ONE_NAME_1:
					{
						display_write_text("S. Ram", NO_LINE_TWO,10);
					} break;

					case LEVEL_ONE_NAME_2:
					{
						display_write_text("S. Flotzinger", NO_LINE_TWO,10);
					} break;

					case LEVEL_ONE_NAME_3:
					{
						display_write_text("R. Freingruber", NO_LINE_TWO,10);
					} break;
				}
			} break;
		}
	}
}





/*******************************************************
	Funktion :	Schreibt auf Display einen Text
	Eigenschaften: schreibt Text/Zahl
	bergabewert: Stringpointer, Zahl die angezeigt wird, Basis dieser Zahl (10 oder 16)
	Rckgabewert: -
********************************************************/
void display_write_text(char *text, uint8_t value, uint8_t basis)
{
	char buffer_value[6];				// Um Value in String umzuwandeln (6 weil 6-stellige Zahl + Nullterminierung)
	itoa(value, buffer_value, basis);	// Wandle um, Basis ist dabei frie whlbar (10 oder 16 z.b.)

	// Lschen der vorhandenen Werte
	zeile=0;
	spalte=0;
	lcd_string(PSTR("                "));

	zeile=1;
	spalte=0;
	lcd_string(PSTR("                "));

	// Ausgabe der erhaltenen Werte
	zeile=0;
	spalte=0;

	lcd_stringr(text);

	zeile=1;
	spalte=0;
	if(value != NO_LINE_TWO)
	{
		lcd_stringr(buffer_value);
	}
}
